@@ -1,23 +1,62 @@
 # shadowstep/element/base.py
+from __future__ import annotations
 
-import re
+import datetime
 import inspect
 import logging
-logger = logging.getLogger(__name__)
+import re
 import typing
-from typing import Union, Optional, Tuple, Dict
-
-from selenium.types import WaitExcTypes
-from selenium.webdriver.support import expected_conditions as EC
-from selenium.common.exceptions import WebDriverException, TimeoutException, NoSuchElementException, \
-    InvalidSessionIdException
-from selenium.webdriver.support.wait import WebDriverWait
+from typing import Dict, Optional, Tuple, Union
 
 from appium.webdriver.webdriver import WebDriver
-from appium.webdriver import WebElement
+from appium.webdriver.webelement import WebElement
+from selenium.common.exceptions import (
+    InvalidSessionIdException,
+    NoSuchElementException,
+    TimeoutException,
+    WebDriverException,
+)
+from selenium.types import WaitExcTypes
+from selenium.webdriver.support import expected_conditions as EC
+from selenium.webdriver.support.wait import WebDriverWait
 
 from shadowstep.base import WebDriverSingleton
 from shadowstep.utils.locator_converter import LocatorConverter
+from shadowstep.utils.utils import get_current_func_name
+
+logger = logging.getLogger(__name__)
+
+
+class ShadowstepNoSuchElementError(NoSuchElementException):
+    def __init__(self, msg: str, screen: str, stacktrace: list,
+                 locator: Union[Tuple, Dict[str, str]]):
+        super().__init__(msg, screen, stacktrace)
+        self.locator = locator
+        self.msg = msg
+        self.screen = screen
+        self.stacktrace = stacktrace
+
+    def __str__(self):
+        return f"ShadowstepNoSuchElementError: Locator: {self.locator} \n Message: {self.msg} \n Stacktrace: {self.stacktrace}"
+
+
+class ShadowstepTimeoutException(TimeoutException):
+    """Custom timeout exception with additional context."""
+
+    def __init__(self, msg: str = None, screen: str = None, stacktrace: list = None,
+                 locator: Union[Tuple, Dict[str, str]] = None, driver=None):
+        super().__init__(msg, screen, stacktrace)
+        self.locator = locator
+        self.driver = driver
+        self.timestamp = datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
+
+    def __str__(self):
+        return (f"ShadowstepTimeoutException\n"
+                f"Timestamp: {self.timestamp}\n"
+                f"Message: {self.msg}\n"
+                f"Locator: {self.locator}\n"
+                f"Current URL: {self.driver.current_url if self.driver else 'N/A'}\n"
+                f"Stacktrace:\n{''.join(self.stacktrace) if self.stacktrace else 'N/A'}")
 
 
 class ElementBase:
@@ -26,8 +65,8 @@ class ElementBase:
     """
 
     def __init__(self,
-                 locator: Union[Tuple, Dict[str, str], str, WebElement] = None,
-                 base=None,
+                 locator: tuple[str, str] | dict[str, str] | str | WebElement,
+                 base: "Shadowstep",
                  timeout: float = 30,
                  poll_frequency: float = 0.5,
                  ignored_exceptions: typing.Optional[WaitExcTypes] = None,
@@ -68,7 +107,7 @@ class ElementBase:
             Union[WebElement, None]
                 The located web element, or None if not found.
         """
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
+        self.logger.debug(f"{get_current_func_name()}")
         self._get_driver()
         if isinstance(locator, WebElement):
             return locator
@@ -80,27 +119,44 @@ class ElementBase:
             self.id = element.id
             return element
         except NoSuchElementException as error:
-            self.logger.error(f"{inspect.currentframe().f_code.co_name} {locator=} {error}")
-            raise
+            self.logger.debug(f"{get_current_func_name()} {locator=} {error}")
+            raise ShadowstepNoSuchElementError(
+                msg=error.msg,
+                screen=error.screen,
+                stacktrace=error.stacktrace,
+                locator=locator
+            )
         except TimeoutException as error:
-            self.logger.error(f"{inspect.currentframe().f_code.co_name} {locator=} {error}")
+            self.logger.debug(f"{get_current_func_name()} {locator=} {error}")
             for stack in error.stacktrace:
                 if 'NoSuchElementError' in stack:
-                    raise NoSuchElementException(msg=error.msg,
-                                                 screen=error.screen,
-                                                 stacktrace=error.stacktrace)
-            raise
+                    raise ShadowstepNoSuchElementError(
+                        msg=error.msg,
+                        screen=error.screen,
+                        stacktrace=error.stacktrace,
+                        locator=locator
+                    )
+            raise ShadowstepTimeoutException(
+                msg=f"Timeout waiting for element with locator: {locator}. Original: {error.msg}",
+                screen=error.screen,
+                stacktrace=error.stacktrace,
+                locator=locator,
+                driver=self.driver
+            )
         except InvalidSessionIdException as error:
-            self.logger.error(f"{inspect.currentframe().f_code.co_name} {locator=} {error}")
-            return None
+            self.logger.debug(f"{get_current_func_name()} {locator=} {error}")
+            raise
         except WebDriverException as error:
-            self.logger.error(f"{inspect.currentframe().f_code.co_name} {locator} {error}")
-            return None
+            from icecream import ic
+            ic()
+            ic("DEBUG INSTRUMENTATION PROCESS CRUSHED (INVESTIGATION)")
+            self.logger.debug(f"{get_current_func_name()} {locator=} {error}")
+            raise
 
     def handle_locator(self,
                        locator: Union[Tuple[str, str], Dict[str, str], str, WebElement],
                        contains: bool = False) -> Optional[Tuple[str, str]]:
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
+        self.logger.debug(f"{get_current_func_name()}")
         if isinstance(locator, tuple):
             by, value = locator
             # Удаляем части типа [@attr='null']
@@ -126,7 +182,7 @@ class ElementBase:
             Union[Tuple, None]
                 The XPath locator as a tuple, or None if there was an error.
         """
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
+        self.logger.debug(f"{get_current_func_name()}")
         if 'class' not in locator:
             xpath = "//*"
         else:
@@ -154,5 +210,5 @@ class ElementBase:
             WebDriverSingleton
                 The WebDriver instance.
         """
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
+        self.logger.debug(f"{get_current_func_name()}")
         self.driver = WebDriverSingleton.get_driver()
