@@ -1,37 +1,40 @@
 import subprocess
 import time
-from typing import Generator
+from collections.abc import Generator
+from typing import Any
+
 import pytest
 from selenium.common import NoSuchElementException
 
 from shadowstep.element.element import Element, GeneralElementException
-from shadowstep.elements.elements import Elements
 from shadowstep.shadowstep import Shadowstep
 
 
 class TestElement:
 
-    def test_get_element_positive(self, app: Shadowstep, android_settings, android_settings_recycler):
+    def test_get_element_positive(self, app: Shadowstep, stability: None, android_settings_open_close,
+                                  android_settings_recycler: Element):
         inner_element = android_settings_recycler.get_element(locator={'text': 'Network & internet'})
         assert isinstance(inner_element, Element)
-        assert inner_element.get_attribute('package') == 'com.android.settings'
-        assert inner_element.get_attribute('class') == 'android.widget.TextView'
-        assert inner_element.get_attribute('resource-id') == 'android:id/title'
+        assert inner_element.package == 'com.android.settings'
+        assert inner_element.class_ == 'android.widget.TextView'
+        assert inner_element.resource_id == 'android:id/title'
 
-    def test_get_element_contains(self, app: Shadowstep, android_settings, android_settings_recycler):
+    def test_get_element_contains(self, app: Shadowstep, stability: None, android_settings_open_close,
+                                  android_settings_recycler: Element):
         inner_element = android_settings_recycler.get_element(locator={'text': 'ork & int'},
-                                                   contains=True)
+                                                              contains=True)
         assert inner_element.contains
-        assert inner_element.get_attribute('text') == 'Network & internet'
+        assert inner_element.text == 'Network & internet'
 
-    def test_get_element_repeated_search(self, app: Shadowstep):
+    def test_get_element_repeated_search(self, app: Shadowstep, stability: None):
         element1 = app.get_element(locator={'content-desc': 'Phone'})
         element2 = app.get_element(locator={'content-desc': 'Phone'})
         assert element1 is not None
         assert element2 is not None
         assert element1.locator == element2.locator
 
-    def test_get_element_disconnected(self, app: Shadowstep):
+    def test_get_element_disconnected(self, app: Shadowstep, stability: None):
         app.disconnect()
         assert not app.is_connected()
         element = app.get_element(locator={'content-desc': 'Phone'})
@@ -40,25 +43,25 @@ class TestElement:
         assert isinstance(element, Element)
         assert element.locator == {'content-desc': 'Phone'}
 
-    def test_get_elements(self, app: Shadowstep, android_settings, android_settings_recycler):
+    def test_get_elements(self, app: Shadowstep, stability: None, android_settings_open_close,
+                          android_settings_recycler: Element):
         inner_elements = android_settings_recycler.get_elements(locator={'resource-id': 'android:id/title'})
-        assert isinstance(inner_elements, Elements)
+        assert isinstance(inner_elements, list)
         for inner_element in inner_elements:
             app.logger.info(f"{inner_element.text=}")
             assert isinstance(inner_element, Element)
             assert inner_element.get_attribute('resource-id') == 'android:id/title'
 
-
-    def test_get_attributes(self, app: Shadowstep):
+    def test_get_attributes(self, app: Shadowstep, stability: None):
         element = app.get_element(locator={'package': 'com.android.launcher3',
                                            'class': 'android.view.ViewGroup',
                                            'resource-id': 'com.android.launcher3:id/hotseat',
                                            })
         attrs = element.get_attributes()
         assert isinstance(attrs, dict)
-        assert 'bounds' in attrs.keys()
+        assert 'bounds' in attrs
 
-    def test_get_parent(self, app: Shadowstep):
+    def test_get_parent(self, app: Shadowstep, stability: None):
         child = app.get_element(locator={'content-desc': 'Phone'})
         parent = child.get_parent()
         assert isinstance(parent, Element)
@@ -67,7 +70,7 @@ class TestElement:
         parent = child.get_parent()
         assert 'com.android.launcher3:id/launcher' in parent.get_attribute('resource-id')
 
-    def test_get_parents(self, app: Shadowstep):
+    def test_get_parents(self, app: Shadowstep, stability: None):
         element = app.get_element(locator={'content-desc': 'Phone'})
         parents = element.get_parents()
         assert isinstance(parents, Generator)
@@ -78,7 +81,7 @@ class TestElement:
         assert count > 0
         app.adb.press_home()
 
-    def test_get_sibling(self, app: Shadowstep):
+    def test_get_sibling(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         sibling = el.get_sibling({'content-desc': 'WebView Browser Tester'})
         assert isinstance(sibling, Element)
@@ -89,8 +92,11 @@ class TestElement:
         assert isinstance(sibling, Element)
         assert 'WebView Browser Tester' in sibling.get_attribute('text')
 
-    def test_get_siblings(self, app: Shadowstep, android_settings, android_settings_recycler):
-        el = android_settings_recycler.get_element({'resource-id': 'com.android.settings:id/recycler_view'}).get_element({'class': 'android.widget.LinearLayout'})
+    def test_get_siblings(self, app: Shadowstep, stability: None, android_settings_open_close,
+                          android_settings_recycler: Element):
+        el = android_settings_recycler.get_element(
+            {'resource-id': 'com.android.settings:id/recycler_view'}).get_element(
+            {'class': 'android.widget.LinearLayout'})
         siblings = el.get_siblings()
         assert isinstance(siblings, Generator)
         count = 0
@@ -104,14 +110,14 @@ class TestElement:
         assert len(bounds) > 3
         assert count > 0
 
-    def test_tap(self, app: Shadowstep) -> None:
+    def test_tap(self, app: Shadowstep, stability: None):
         element = app.get_element(locator={'content-desc': 'Phone'})
         element.tap()
         time.sleep(3)
         response = str(subprocess.check_output('adb shell "dumpsys window windows | grep -E \'mSurface\'"'))
         assert "com.android.dialer" in response
 
-    def test_tap_duration(self, app: Shadowstep):
+    def test_tap_duration(self, app: Shadowstep, stability: None):
         phone = app.get_element(locator={'content-desc': 'Phone'})
         phone.tap(duration=3000)
         bubble = app.get_element(locator={'package': 'com.android.launcher3',
@@ -125,7 +131,7 @@ class TestElement:
         assert phone_info_title.get_attribute('content-desc') == 'App info'
         assert phone_info_storage.get_attribute('text') == 'Storage & cache'
 
-    def test_tap_no_such_driver_exception(self, app: Shadowstep):
+    def test_tap_no_such_driver_exception(self, app: Shadowstep, stability: None):
         app.disconnect()
         assert not app.is_connected()
         element = app.get_element(locator={'content-desc': 'Phone'})
@@ -135,7 +141,7 @@ class TestElement:
         response = str(subprocess.check_output('adb shell "dumpsys window windows | grep -E \'mSurface\'"'))
         assert "com.android.dialer" in response
 
-    def test_tap_invalid_session_id_exception(self, app: Shadowstep):
+    def test_tap_invalid_session_id_exception(self, app: Shadowstep, stability: None):
         app.driver.session_id = '12345'
         element = app.get_element(locator={'content-desc': 'Phone'})
         element.tap()
@@ -144,17 +150,17 @@ class TestElement:
         response = str(subprocess.check_output('adb shell "dumpsys window windows | grep -E \'mSurface\'"'))
         assert "com.android.dialer" in response
 
-    def test_tap_no_such_element_exception(self, app: Shadowstep):
+    def test_tap_no_such_element_exception(self, app: Shadowstep, stability: None):
         try:
             element = app.get_element(locator={'content-desc': 'no_such_element'})
             element.tap()
         except Exception as error:
             assert isinstance(error, NoSuchElementException)
 
-    def test_tap_stale_element_reference_exception(self, app: Shadowstep):
+    def test_tap_stale_element_reference_exception(self, app: Shadowstep, stability: None):
         pass  # don't know how to catch
 
-    def test_tap_invalid_element_state_exception(self, app: Shadowstep):
+    def test_tap_invalid_element_state_exception(self, app: Shadowstep, stability: None):
         pass  # don't know how to catch
 
     @pytest.mark.parametrize("params", [
@@ -164,7 +170,7 @@ class TestElement:
                      'resource-id': 'com.android.quicksearchbox:id/search_widget_text'}},  # Локатор
         {"direction": 0, "distance": 1000},  # Вверх
     ])
-    def test_tap_and_move(self, app: Shadowstep, params):
+    def test_tap_and_move(self, app: Shadowstep, stability: None, params: Any):
         element = app.get_element(locator={"content-desc": "Phone"})
         target_element = app.get_element(locator={'resource-id': 'com.android.launcher3:id/search_container_all_apps'})
         element.tap_and_move(**params)
@@ -172,15 +178,14 @@ class TestElement:
         assert 'Search apps' in target_element.get_attribute(name='text')
         assert isinstance(element, Element)
 
-    # FIXME почему-то один проходит а в связке - нет
-    def test_click(self, app: Shadowstep):
+    def test_click(self, app: Shadowstep, stability: None):
         element = app.get_element(locator={'content-desc': 'Phone'})
         element.click()
         time.sleep(5)
         response = str(subprocess.check_output('adb shell "dumpsys window windows | grep -E \'mSurface\'"'))
         assert "com.android.dialer" in response
 
-    def test_click_duration(self, app: Shadowstep):
+    def test_click_duration(self, app: Shadowstep, stability: None):
         phone = app.get_element(locator={'content-desc': 'Phone'})
         phone.click(duration=3000)
         bubble = app.get_element(locator={'package': 'com.android.launcher3',
@@ -195,7 +200,7 @@ class TestElement:
         assert phone_info_title.get_attribute('content-desc') == 'App info'
         assert phone_info_storage.get_attribute('text') == 'Storage & cache'
 
-    def test_click_double(self, app: Shadowstep):
+    def test_click_double(self, app: Shadowstep, stability: None):
         search = app.get_element(locator={'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         search.click_double()
         time.sleep(5)
@@ -203,23 +208,28 @@ class TestElement:
         app.terminal.past_text(text="some_text")
         assert 'some_text' in search_src_text.get_attribute('text')
 
-    def test_drag(self, app: Shadowstep):
+    def test_drag(self, app: Shadowstep, stability: None):
         app.terminal.press_home()
         time.sleep(10)
-        dev_settings = app.get_element(locator={"content-desc": "Appium Settings"})
+        messaging_1 = app.get_element(locator={"content-desc": "Messaging"})
+        messaging_1.timeout = 1
+        m1_center_x, m1_center_y, = messaging_1.get_center()
         search = app.get_element(locator={'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         assert 'com.android.quicksearchbox' in search.get_attribute('package')
         end_x, end_y = search.get_center()
         app.get_element(locator={"content-desc": "Phone"}).tap_and_move(x=100, y=500)
         time.sleep(1)
-        dev_settings.drag(end_x=100, end_y=500)
+        messaging_1.drag(end_x=100, end_y=500)
         time.sleep(1)
-        dev_settings.drag(end_x=end_x, end_y=end_y)
-        dev_settings.timeout = 5
+        messaging_1.drag(end_x=end_x, end_y=end_y)
+        messaging_1.timeout = 5
         time.sleep(1)
-        assert not dev_settings.is_visible()
+        assert not messaging_1.is_visible()
+        app.get_element(locator={"content-desc": "Phone"}).tap_and_move(x=100, y=500)
+        messaging_2 = app.get_element(locator={"content-desc": "Messaging"})
+        messaging_2.drag(end_x=m1_center_x, end_y=m1_center_y)
 
-    def test_is_within_screen(self, app: Shadowstep):
+    def test_is_within_screen(self, app: Shadowstep, stability: None):
         phone = app.get_element(locator={"content-desc": "Phone"}, timeout=5)
         search = app.get_element(locator={'resource-id': 'com.android.quicksearchbox:id/search_widget_text'}, timeout=5)
         assert search.is_visible() is True
@@ -229,7 +239,7 @@ class TestElement:
         assert phone.is_visible() is False
         assert search.is_visible() is False
 
-    def test_fling(self, app: Shadowstep):
+    def test_fling(self, app: Shadowstep, stability: None):
         element = app.get_element(locator={"content-desc": "Phone"})
         target_element = app.get_element(locator={'content-desc': 'Do Not Disturb.'})
         element.fling_up(speed=2000)
@@ -237,7 +247,7 @@ class TestElement:
         assert 'Off' in target_element.get_attribute(name='text')
         assert isinstance(element, Element)
 
-    def test_scroll(self, app: Shadowstep):
+    def test_scroll(self, app: Shadowstep, stability: None):
         settings_recycler = app.get_element(
             locator={'resource-id': 'com.android.settings:id/main_content_scrollable_container'})
         settings_network = app.get_element(locator={'text': 'Network & internet',
@@ -252,7 +262,7 @@ class TestElement:
         assert 'About phone' in settings_about_phone.get_attribute('text')
         app.terminal.close_app(package='com.android.settings')
 
-    def test_scroll_to_bottom(self, app: Shadowstep, android_settings):
+    def test_scroll_to_bottom(self, app: Shadowstep, stability: None, android_settings_open_close):
         settings_recycler = app.get_element(
             locator={'resource-id': 'com.android.settings:id/main_content_scrollable_container'})
         settings_network = app.get_element(locator={'text': 'Network & internet',
@@ -270,7 +280,7 @@ class TestElement:
         app.logger.info(f"{settings_about_phone.get_attributes()=}")
         app.terminal.close_app(package='com.android.settings')
 
-    def test_scroll_to_top(self, app: Shadowstep, android_settings):
+    def test_scroll_to_top(self, app: Shadowstep, stability: None, android_settings_open_close):
         settings_recycler = app.get_element(
             locator={'resource-id': 'com.android.settings:id/main_content_scrollable_container'})
         settings_network = app.get_element(locator={'text': 'Network & internet',
@@ -288,7 +298,7 @@ class TestElement:
         assert 'Network & internet' in settings_network.get_attribute('text')
         app.terminal.close_app(package='com.android.settings')
 
-    def test_scroll_to_element(self, app: Shadowstep, android_settings):
+    def test_scroll_to_element(self, app: Shadowstep, stability: None, android_settings_open_close):
         settings_recycler = app.get_element(
             locator={'resource-id': 'com.android.settings:id/main_content_scrollable_container'})
         settings_network = app.get_element(locator={'text': 'Network & internet',
@@ -306,63 +316,66 @@ class TestElement:
         assert 'Network & internet' in settings_network.get_attribute('text')
         app.terminal.close_app(package='com.android.settings')
 
-    def test_get_center(self, app: Shadowstep):
+    def test_get_center(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         center = el.get_center()
+        left, top, right, bottom = map(int, el.bounds.strip("[]").replace("][", ",").split(","))
+        x = int((left + right) / 2)
+        y = int((top + bottom) / 2)
         assert isinstance(center, tuple) and len(center) == 2
-        assert center == (126, 2184)
-
-    def test_get_coordinates(self, app: Shadowstep):
+        assert center == (x, y)
+        
+    def test_get_coordinates(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         coords = el.get_coordinates()
+        left, top, right, bottom = map(int, el.bounds.strip("[]").replace("][", ",").split(","))
         assert isinstance(coords, tuple) and len(coords) == 4
-        assert coords == (23, 2100, 230, 2269)
+        assert coords == (left, top, right, bottom)
 
-    def test_get_attribute(self, app: Shadowstep):
+    def test_get_attribute(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert el.get_attribute('content-desc') == 'Phone'
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_get_property(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_get_property(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         prop = el.get_property('enabled')
-        assert isinstance(prop, (str, bool, dict, type(None)))
+        assert isinstance(prop, (str, bool, dict, type(None)))  # noqa: UP038
 
-    def test_get_dom_attribute(self, app: Shadowstep):
+    def test_get_dom_attribute(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert el.get_dom_attribute('class') == 'android.widget.TextView'
 
-    def test_is_displayed(self, app: Shadowstep):
+    def test_is_displayed(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert isinstance(el.is_displayed(), bool)
         assert el.is_displayed()
 
-    def test_is_selected(self, app: Shadowstep):
+    def test_is_selected(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert isinstance(el.is_selected(), bool)
         assert not el.is_selected()
 
-    def test_is_enabled(self, app: Shadowstep):
+    def test_is_enabled(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert isinstance(el.is_enabled(), bool)
         assert el.is_enabled()
 
-    def test_is_contains(self, app: Shadowstep):
+    def test_is_contains(self, app: Shadowstep, stability: None):
         el = app.get_element({'resource-id': 'com.android.launcher3:id/hotseat'})
         assert el.is_contains({'content-desc': 'Phone'}) is True
 
-    def test_tag_name(self, app: Shadowstep):
+    def test_tag_name(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert isinstance(el.tag_name, str)
         assert el.tag_name == 'Phone'
 
-    def test_text(self, app: Shadowstep):
+    def test_text(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert isinstance(el.text, str)
         assert el.text == 'Phone'
 
-    # FIXME почему-то один проходит а в связке - нет
-    def test_clear(self, app: Shadowstep):
+    def test_clear(self, app: Shadowstep, stability: None):
         el = app.get_element({'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         el.tap()
         app.terminal.past_text('some_text')
@@ -372,8 +385,8 @@ class TestElement:
         el.clear()
         assert el.text == ''
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_set_value(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_set_value(self, app: Shadowstep, stability: None):
         el = app.get_element({'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         el.tap()
         time.sleep(3)
@@ -382,7 +395,7 @@ class TestElement:
         assert "test123" in el.text
         el.clear()
 
-    def test_send_keys(self, app: Shadowstep):
+    def test_send_keys(self, app: Shadowstep, stability: None):
         el = app.get_element({'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         el.tap()
         time.sleep(3)
@@ -391,50 +404,50 @@ class TestElement:
         assert "abc" in el.text
         el.clear()
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_submit(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_submit(self, app: Shadowstep, stability: None):
         el = app.get_element({'resource-id': 'com.android.quicksearchbox:id/search_widget_text'})
         el.submit()  # Не всегда валидно, но для теста вызова достаточно
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_shadow_root(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_shadow_root(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         try:
             sr = el.shadow_root
             assert sr is not None
         except Exception as e:
-            assert isinstance(e, (NoSuchElementException, AttributeError))
+            assert isinstance(e, (NoSuchElementException, AttributeError))  # noqa: UP038
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_location_once_scrolled_into_view(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_location_once_scrolled_into_view(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         loc = el.location_once_scrolled_into_view
         assert 'x' in loc and 'y' in loc
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_size_location_rect(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_size_location_rect(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         assert 'width' in el.size and 'height' in el.size
         assert 'x' in el.location and 'y' in el.location
         assert all(k in el.rect for k in ('x', 'y', 'width', 'height'))
 
-    @pytest.mark.xfail(reason="Method is not implemented in UiAutomator2")
-    def test_value_of_css_property(self, app: Shadowstep):
+    @pytest.mark.skip(reason="Method is not implemented in UiAutomator2")
+    def test_value_of_css_property(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         value = el.value_of_css_property("display")
         assert isinstance(value, str)
 
-    def test_screenshot_as_base64(self, app: Shadowstep):
+    def test_screenshot_as_base64(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         ss = el.screenshot_as_base64
         assert isinstance(ss, str)
 
-    def test_screenshot_as_png(self, app: Shadowstep):
+    def test_screenshot_as_png(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         ss = el.screenshot_as_png
         assert isinstance(ss, bytes)
 
-    def test_save_screenshot(self, tmp_path, app: Shadowstep):
+    def test_save_screenshot(self, tmp_path: Any, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'Phone'})
         filepath = tmp_path / "test_element.png"
         assert el.save_screenshot(str(filepath)) is True
@@ -442,31 +455,28 @@ class TestElement:
         filepath.unlink()
         assert not filepath.exists()
 
-    def test_shadow_root_error(self, app: Shadowstep):
+    def test_shadow_root_error(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'non_existing'})
         with pytest.raises(GeneralElementException):
             _ = el.shadow_root
 
-    def test_get_attribute_no_such_element(self, app: Shadowstep):
+    def test_get_attribute_no_such_element(self, app: Shadowstep, stability: None):
         el = app.get_element({'content-desc': 'non_existing'})
         with pytest.raises(NoSuchElementException):
             el.get_attribute("text")
 
-    def test_scroll_to_element_not_found(self, app):
+    def test_scroll_to_element_not_found(self, app: Shadowstep, stability: None):
         app.terminal.start_activity(package="com.android.settings", activity=".Settings")
         container = app.get_element({'resource-id': 'com.android.settings:id/main_content_scrollable_container'})
         with pytest.raises(GeneralElementException):
             container.scroll_to_element(locator={'text': 'Element That Does Not Exist'})
 
-    def test_get_cousin(self, app, android_settings):
+    def test_get_cousin(self, app: Shadowstep, stability: None, android_settings_open_close):
         app.get_element({'text': 'Network & internet'}).tap()
         switcher = app.get_element({'text': 'Airplane mode'}).get_cousin({'resource-id': 'android:id/switch_widget'})
         assert switcher.get_attribute('class') == 'android.widget.Switch'
 
-    def test_get_cousin_depth(self, app, android_settings):
+    def test_get_cousin_depth(self, app: Shadowstep, stability: None, android_settings_open_close):
         app.get_element({'text': 'Network & internet'}).tap()
         switcher = app.get_element({'text': 'Airplane mode'}).get_cousin({'resource-id': 'android:id/switch_widget'}, 5)
         assert switcher.get_attribute('class') == 'android.widget.Switch'
-
-
-
